#define _CRT_SECURE_NO_WARNINGS
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>


typedef unsigned __int8  UI8;
typedef   signed __int8  SI8;
typedef unsigned __int16 UI16;
typedef   signed __int16 SI16;
typedef unsigned __int32 UI32;
typedef   signed __int32 SI32;
typedef unsigned __int64 UI64;
typedef   signed __int64 SI64;


typedef SI32 (NTAPI *PFN_NT_QUERY_PERFORMANCE_COUNTER)(OUT PLARGE_INTEGER PerformanceCounter, OUT PLARGE_INTEGER PerformanceFrequency OPTIONAL);


int __cdecl main(void)
{
	LARGE_INTEGER PerformanceCounter, PerformanceFrequency;
	HMODULE Ntdll;
	PFN_NT_QUERY_PERFORMANCE_COUNTER NtQueryPerformanceCounter;
	UI64 StartTime, StartTick;
	SI32 Status;

	Ntdll = GetModuleHandleW(L"ntdll.dll");
	NtQueryPerformanceCounter = (PFN_NT_QUERY_PERFORMANCE_COUNTER) GetProcAddress(Ntdll, "NtQueryPerformanceCounter");

	// Take an initial sample of two time sources: the TSC and the HPET
	Status = NtQueryPerformanceCounter(&PerformanceCounter, &PerformanceFrequency);
	if (Status < 0)
	{
		printf("NtQueryPerformanceCounter(): %08X\n", Status);
		return 0;
	}

	StartTick = __rdtsc();
	StartTime = PerformanceCounter.QuadPart;

	printf("NtQueryPerformanceCounter(): CTR=0x%016llX FRQ=0x%016llX\r\n", PerformanceCounter.QuadPart, PerformanceFrequency.QuadPart);

	for(size_t i = 0; i < 60; i++)
	{
		UI64 EndTime, EndTick, DeltaTime, DeltaTick, TickRate;

		// Sleep for a while.
		Sleep(1000);

		// Sample the two clocks again. N.B. rdtsc is sampled *first* this time.
		EndTick = __rdtsc();
		Status = NtQueryPerformanceCounter(&PerformanceCounter, &PerformanceFrequency);
		if (Status < 0)
		{
			printf("NtQueryPerformanceCounter(): %08X\n", Status);
			return 0;
		}

		EndTime = PerformanceCounter.QuadPart;

		printf("NtQueryPerformanceCounter(): CTR=0x%016llX FRQ=0x%016llX\r\n", PerformanceCounter.QuadPart, PerformanceFrequency.QuadPart);

		DeltaTime = EndTime - StartTime;
		DeltaTick = EndTick - StartTick;
		TickRate = (DeltaTick * 10000000ull) / DeltaTime;

		printf("[%2u] TIME=+%llu.%03u  TICKS=+%llu  (%llu ticks/s)\r\n", i, DeltaTime / 10000000ull, (unsigned int)(DeltaTime % 10000000ull) / 10000, DeltaTick, TickRate);

		EndTick += DeltaTick;
		EndTime += DeltaTime;
	}

	return 0;
}
